import 'package:flutter/widgets.dart';
import 'models.dart';

/// Standard pair data type. Could be seen as a tuple.
class Pair<T, D> {
  Pair(this.first, this.second);

  final T first;
  final D second;

  @override
  String toString() => 'Pair[$first, $second]';
}

/// The `GraphController` holds data to be drawn on the graph. It also notifies
/// listeners on selection change. At least one group of data is required to draw
/// the graph.
class GraphController extends ValueNotifier<Pair<DataGroup, Data>> {
  List<DataGroup> groups;

  Data maxValue;

  GraphController(this.groups) : super(null);

  /// This should be called right before it is passed to a [MinimalGraph].
  /// It makes a default selection to the last item in the combined groups data and sets
  /// the maximum value of all data combined.
  ///
  /// If this behaviour is not intended, you can manually use [select] to set your
  /// custom default.
  void initialize() {
    value = Pair(this.groups.last, this.groups.last.data.last);

    final _chart = <Data>[];

    this.groups.forEach((_group) {
      _chart.addAll(_group.data);
    });

    maxValue = _chart.reduce((v1, v2) {
      return v1.value > v2.value ? v1 : v2;
    });
  }

  void select(Data dataItem) {
    final containerGroup = this.groups.firstWhere((dg) {
      return dg.data.contains(dataItem);
    });

    value = Pair(containerGroup, dataItem);
  }

  DataGroup getBelongingGroup(Data data) {
    return groups.firstWhere((it) {
      return it.data.contains(data);
    });
  }

  bool isActive(Data dataItem) {
    return this.value.first.data.contains(dataItem);
  }
}
